@@ -128,6 +128,7 @@ sub accept_loop {
                     'psgi.nonblocking'  => Plack::Util::FALSE,
                     'psgix.input.buffered' => Plack::Util::TRUE,
                     'psgix.io'          => $conn,
+                    'psgix.harakiri'    => 1,
                 };
 
                 # no need to take care of pipelining since this module is a HTTP/1.0 server
@@ -136,8 +137,12 @@ sub accept_loop {
                     $may_keepalive = undef;
                 }
                 $is_keepalive = ($req_count != 1) ? 1 : 0;
-                $self->handle_connection($env, $conn, $app, $may_keepalive, $req_count != 1)
-                    or last;
+                my $use_keepalive = $self->handle_connection($env, $conn, $app, $may_keepalive, $req_count != 1);
+                if ($env->{'psgix.harakiri.commit'}) {
+                    $conn->close;
+                    return;
+                }
+                $use_keepalive or last;
                 # TODO add special cases for clients with broken keep-alive support, as well as disabling keep-alive for HTTP/1.0 proxies
             }
             $conn->close;
